@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2011  Université de Bordeaux 1
+ * Copyright (C) 2011-2012                                Inria
+ * Copyright (C) 2012-2013,2015-2017                      CNRS
+ * Copyright (C) 2010-2011,2013-2014                      Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -19,9 +21,12 @@
 /* #define _externC extern "C" */
 
 #include <stencil.h>
+#ifdef __APPLE__
+#include <OpenCL/cl.h>
+#else
 #include <CL/cl.h>
+#endif
 #include <starpu.h>
-#include <starpu_opencl.h>
 
 #define str(x) #x
 
@@ -75,35 +80,29 @@ opencl_life_init(void)
 
 void opencl_life_free(void)
 {
-  starpu_opencl_unload_opencl(&program);
+  int ret = starpu_opencl_unload_opencl(&program);
+  STARPU_CHECK_RETURN_VALUE(ret, "starpu_opencl_unload_opencl");
 }
 
 void
 opencl_life_update_host(int bz, const TYPE *old, TYPE *newp, int nx, int ny, int nz, int ldy, int ldz, int iter)
 {
-	unsigned max_parallelism = 512;
-	unsigned threads_per_dim_x = max_parallelism;
-	while (threads_per_dim_x / 2 >= nx)
-		threads_per_dim_x /= 2;
-	unsigned threads_per_dim_y = max_parallelism / threads_per_dim_x;
-	while (threads_per_dim_y / 2 >= ny)
-		threads_per_dim_y /= 2;
 #if 0
-	unsigned threads_per_dim_z = 4;
-	size_t dimBlock[] = {threads_per_dim_x, threads_per_dim_y, threads_per_dim_z};
-	size_t dimGrid[] = {nx / threads_per_dim_x, ny / threads_per_dim_y, nz / threads_per_dim_z};
+	size_t dim[] = {nx, ny, nz};
 #else
-	size_t dimBlock[] = {threads_per_dim_x, threads_per_dim_y, 1};
-	size_t dimGrid[] = {((nx + threads_per_dim_x-1) / threads_per_dim_x)*threads_per_dim_x, ((ny + threads_per_dim_y-1) / threads_per_dim_y)*threads_per_dim_y, 1};
+	size_t dim[] = {nx, ny, 1};
 #endif
 
   int devid,id;
-  id = starpu_worker_get_id();
+  cl_int err;
+
+  id = starpu_worker_get_id_check();
   devid = starpu_worker_get_devid(id);
 
   cl_kernel kernel;
   cl_command_queue cq;
-  starpu_opencl_load_kernel(&kernel, &cq, &program, "life_update", devid);
+  err = starpu_opencl_load_kernel(&kernel, &cq, &program, "life_update", devid);
+  if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 
   clSetKernelArg(kernel, 0, sizeof(bz), &bz);
   clSetKernelArg(kernel, 1, sizeof(old), &old);
@@ -115,9 +114,6 @@ opencl_life_update_host(int bz, const TYPE *old, TYPE *newp, int nx, int ny, int
   clSetKernelArg(kernel, 7, sizeof(ldz), &ldz);
   clSetKernelArg(kernel, 8, sizeof(iter), &iter);
 
-  cl_event ev;
-  clEnqueueNDRangeKernel(cq, kernel, 3, NULL, dimGrid, dimBlock, 0, NULL, &ev);
-  clWaitForEvents(1, &ev);
-  starpu_opencl_collect_stats(ev);
-  clReleaseEvent(ev);
+  err = clEnqueueNDRangeKernel(cq, kernel, 3, NULL, dim, NULL, 0, NULL, NULL);
+  if (err != CL_SUCCESS) STARPU_OPENCL_REPORT_ERROR(err);
 }
