@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010-2012 University of Bordeaux
- * Copyright (C) 2012 CNRS
- * Copyright (C) 2012 Vincent Danjean <Vincent.Danjean@ens-lyon.org>
+ * Copyright (C) 2011-2012                                Inria
+ * Copyright (C) 2012,2015,2017                           CNRS
+ * Copyright (C) 2010-2012, 2018                                Université de Bordeaux
+ * Copyright (C) 2012                                     Vincent Danjean
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,12 +25,13 @@
  *
  * \param[in] platform Must be StarPU platform ID or NULL
  */
+CL_API_SUFFIX__VERSION_1_0
 CL_API_ENTRY cl_int CL_API_CALL
 soclGetDeviceIDs(cl_platform_id   platform,
-               cl_device_type   device_type,
-               cl_uint          num_entries,
-               cl_device_id *   devices,
-               cl_uint *        num_devices) CL_API_SUFFIX__VERSION_1_0
+		 cl_device_type   device_type,
+		 cl_uint          num_entries,
+		 cl_device_id *   devices,
+		 cl_uint *        num_devices)
 {
    if( ! _starpu_init )
       socl_init_starpu();
