@@ -1,8 +1,9 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2017  CNRS
- * Copyright (C) 2013 Corentin Salingue
+ * Copyright (C) 2011-2014,2016                           Inria
+ * Copyright (C) 2009-2018                                Université de Bordeaux
+ * Copyright (C) 2010-2017                                CNRS
+ * Copyright (C) 2013                                     Corentin Salingue
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -35,6 +36,7 @@
 #include <core/workers.h>
 #include <core/perfmodel/perfmodel.h>
 #include <core/simgrid.h>
+#include <core/topology.h>
 #include <common/utils.h>
 
 #ifdef STARPU_USE_OPENCL
@@ -45,6 +47,16 @@
 #include <windows.h>
 #endif
 
+#ifdef STARPU_HAVE_HWLOC
+#include <hwloc.h>
+#ifndef HWLOC_API_VERSION
+#define HWLOC_OBJ_PU HWLOC_OBJ_PROC
+#endif
+#if HWLOC_API_VERSION < 0x00010b00
+#define HWLOC_OBJ_NUMANODE HWLOC_OBJ_NODE
+#endif
+#endif
+
 #if defined(HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX) && HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX
 #include <hwloc/cuda.h>
 #endif
@@ -52,6 +64,8 @@
 #define SIZE	(32*1024*1024*sizeof(char))
 #define NITER	32
 
+#define PATH_LENGTH 256
+
 #ifndef STARPU_SIMGRID
 static void _starpu_bus_force_sampling(void);
 #endif
@@ -91,7 +105,7 @@ static double cudadev_timing_htod[STARPU_MAXNODES] = {0.0};
 static double cudadev_latency_htod[STARPU_MAXNODES] = {0.0};
 static double cudadev_timing_dtoh[STARPU_MAXNODES] = {0.0};
 static double cudadev_latency_dtoh[STARPU_MAXNODES] = {0.0};
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 static double cudadev_timing_dtod[STARPU_MAXNODES][STARPU_MAXNODES] = {{0.0}};
 static double cudadev_latency_dtod[STARPU_MAXNODES][STARPU_MAXNODES] = {{0.0}};
 #endif
@@ -238,7 +252,7 @@ static void measure_bandwidth_between_host_and_dev_on_cpu_with_cuda(int dev, int
 	cudaThreadExit();
 }
 
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 static void measure_bandwidth_between_dev_and_dev_cuda(int src, int dst)
 {
 	size_t size = SIZE;
@@ -496,7 +510,11 @@ static int find_numa_node(hwloc_obj_t obj)
 	STARPU_ASSERT(obj);
 	hwloc_obj_t current = obj;
 
-	while (current->depth != HWLOC_OBJ_NODE)
+#if HWLOC_API_VERSION >= 0x00020000
+	while (current->memory_first_child == NULL)
+#else
+	while (current->depth != HWLOC_OBJ_NUMANODE)
+#endif
 	{
 		current = current->parent;
 
@@ -505,8 +523,11 @@ static int find_numa_node(hwloc_obj_t obj)
 		 * we should not use a per-node sampling in that case. */
 		STARPU_ASSERT(current);
 	}
-
-	STARPU_ASSERT(current->depth == HWLOC_OBJ_NODE);
+#if HWLOC_API_VERSION >= 0x00020000
+	current = current->memory_first_child;
+#else
+	STARPU_ASSERT(current->depth == HWLOC_OBJ_NUMANODE);
+#endif
 
 	return current->logical_index;
 }
@@ -520,7 +541,7 @@ static void measure_bandwidth_between_cpus_and_dev(int dev, struct dev_timing *d
 	 * */
 #ifdef STARPU_HAVE_HWLOC
 	int cpu_depth = hwloc_get_type_depth(hwtopology, HWLOC_OBJ_PU);
-	int nnuma_nodes = hwloc_get_nbobjs_by_depth(hwtopology, HWLOC_OBJ_NODE);
+	int nnuma_nodes = hwloc_get_nbobjs_by_depth(hwtopology, HWLOC_OBJ_NUMANODE);
 
 	/* If no NUMA node was found, we assume that we have a single memory
 	 * bank. */
@@ -694,7 +715,7 @@ static void benchmark_all_gpu_devices(void)
 		/* measure bandwidth between Host and Device i */
 		measure_bandwidth_between_host_and_dev(i, cudadev_timing_htod, cudadev_latency_htod, cudadev_timing_dtoh, cudadev_latency_dtoh, cudadev_timing_per_cpu, "CUDA");
 	}
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 	for (i = 0; i < ncuda; i++)
 	{
 		unsigned j;
@@ -779,7 +800,7 @@ static void load_bus_affinity_file_content(void)
 	FILE *f;
 	int locked;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_affinity_path(path, sizeof(path));
 
 	_STARPU_DEBUG("loading affinities from %s\n", path);
@@ -856,7 +877,7 @@ static void write_bus_affinity_file_content(void)
 
 #if defined(STARPU_USE_CUDA) || defined(STARPU_USE_OPENCL)
 	FILE *f;
-	char path[256];
+	char path[PATH_LENGTH];
 	int locked;
 
 	get_affinity_path(path, sizeof(path));
@@ -927,7 +948,7 @@ static void load_bus_affinity_file(void)
 {
 	int res;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_affinity_path(path, sizeof(path));
 
 	res = access(path, F_OK);
@@ -1007,7 +1028,7 @@ static int load_bus_latency_file_content(void)
 	double latency;
 	int locked;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_latency_path(path, sizeof(path));
 
 	_STARPU_DEBUG("loading latencies from %s\n", path);
@@ -1109,7 +1130,7 @@ static void write_bus_latency_file_content(void)
 
 	STARPU_ASSERT(was_benchmarked);
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_latency_path(path, sizeof(path));
 
 	_STARPU_DEBUG("writing latencies to %s\n", path);
@@ -1156,7 +1177,7 @@ static void write_bus_latency_file_content(void)
 			{
 				/* µs */
 #ifdef STARPU_USE_CUDA
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 				if (src && src < ncuda && dst && dst <= ncuda)
 					latency = cudadev_latency_dtod[src][dst];
 				else
@@ -1204,7 +1225,7 @@ static void load_bus_latency_file(void)
 {
 	int res;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_latency_path(path, sizeof(path));
 
 	res = access(path, F_OK);
@@ -1233,7 +1254,7 @@ static int load_bus_bandwidth_file_content(void)
 	double bandwidth;
 	int locked;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_bandwidth_path(path, sizeof(path));
 
 	_STARPU_DEBUG("loading bandwidth from %s\n", path);
@@ -1335,7 +1356,7 @@ static void write_bus_bandwidth_file_content(void)
 
 	STARPU_ASSERT(was_benchmarked);
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_bandwidth_path(path, sizeof(path));
 
 	_STARPU_DEBUG("writing bandwidth to %s\n", path);
@@ -1374,7 +1395,7 @@ static void write_bus_bandwidth_file_content(void)
 				double slowness = 0.0;
 				/* Total bandwidth is the harmonic mean of bandwidths */
 #ifdef STARPU_USE_CUDA
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 				if (src && src <= ncuda && dst && dst <= ncuda)
 					/* Direct GPU-GPU transfert */
 					slowness = cudadev_timing_dtod[src][dst];
@@ -1425,9 +1446,9 @@ static void write_bus_bandwidth_file_content(void)
 
 void starpu_bus_print_filenames(FILE *output)
 {
-	char bandwidth_path[256];
-	char affinity_path[256];
-	char latency_path[256];
+	char bandwidth_path[PATH_LENGTH];
+	char affinity_path[PATH_LENGTH];
+	char latency_path[PATH_LENGTH];
 
 	get_bandwidth_path(bandwidth_path, sizeof(bandwidth_path));
 	get_affinity_path(affinity_path, sizeof(affinity_path));
@@ -1552,7 +1573,7 @@ static void load_bus_bandwidth_file(void)
 {
 	int res;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	get_bandwidth_path(path, sizeof(path));
 
 	res = access(path, F_OK);
@@ -1575,7 +1596,7 @@ static void get_config_path(char *path, size_t maxlen)
 static void check_bus_config_file(void)
 {
         int res;
-        char path[256];
+        char path[PATH_LENGTH];
         struct _starpu_machine_config *config = _starpu_get_machine_config();
 
         get_config_path(path, sizeof(path));
@@ -1661,7 +1682,7 @@ static void check_bus_config_file(void)
 static void write_bus_config_file_content(void)
 {
 	FILE *f;
-	char path[256];
+	char path[PATH_LENGTH];
 	int locked;
 
 	STARPU_ASSERT(was_benchmarked);
@@ -1720,7 +1741,7 @@ void _starpu_simgrid_get_platform_path(int version, char *path, size_t maxlen)
  * - then through all CUDA-CUDA possible transfers again to emit routes.
  */
 
-#if defined(STARPU_USE_CUDA) && defined(HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX) && HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX && defined(HAVE_CUDA_MEMCPY_PEER)
+#if defined(STARPU_USE_CUDA) && defined(HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX) && HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX && defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 
 /* Records, for each PCI link and hub, the maximum bandwidth seen through it */
 struct pci_userdata
@@ -2128,7 +2149,7 @@ static void clean_topology(hwloc_obj_t obj)
 static void write_bus_platform_file_content(int version)
 {
 	FILE *f;
-	char path[256];
+	char path[PATH_LENGTH];
 	unsigned i;
 	const char *speed, *flops, *Bps, *s;
 	char dash;
@@ -2192,7 +2213,7 @@ static void write_bus_platform_file_content(int version)
 	{
 		fprintf(f, "   <host id=\"CUDA%u\" %s=\"2000000000%s\">\n", i, speed, flops);
 		fprintf(f, "     <prop id=\"memsize\" value=\"%llu\"/>\n", (unsigned long long) cuda_size[i]);
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 		fprintf(f, "     <prop id=\"memcpy_peer\" value=\"1\"/>\n");
 #endif
 		fprintf(f, "   </host>\n");
@@ -2276,7 +2297,7 @@ static void write_bus_platform_file_content(int version)
 			cudadev_latency_dtoh[1+i]/1000000., s);
 	}
 	fprintf(f, "\n");
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 	/* Write CUDA/CUDA bandwidths and latencies */
 	for (i = 0; i < ncuda; i++)
 	{
@@ -2297,13 +2318,13 @@ static void write_bus_platform_file_content(int version)
 	}
 #endif
 
-#if defined(HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX) && HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX && defined(HAVE_CUDA_MEMCPY_PEER)
+#if defined(HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX) && HAVE_DECL_HWLOC_CUDA_GET_DEVICE_OSDEV_BY_INDEX && defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 	/* If we have enough hwloc information, write PCI bandwidths and routes */
 	if (!starpu_get_env_number_default("STARPU_PCI_FLAT", 0))
 	{
 		hwloc_topology_t topology;
 		hwloc_topology_init(&topology);
-		hwloc_topology_set_flags(topology, HWLOC_TOPOLOGY_FLAG_IO_DEVICES | HWLOC_TOPOLOGY_FLAG_IO_BRIDGES);
+		_starpu_topology_filter(topology);
 		hwloc_topology_load(topology);
 
 		/* First find paths and record measured bandwidth along the path */
@@ -2369,7 +2390,7 @@ flat_cuda:
 		fprintf(f, "   <route src=\"RAM\" dst=\"%s\" symmetrical=\"NO\"><link_ctn id=\"RAM-%s\"/><link_ctn id=\"Host\"/></route>\n", i_name, i_name);
 		fprintf(f, "   <route src=\"%s\" dst=\"RAM\" symmetrical=\"NO\"><link_ctn id=\"%s-RAM\"/><link_ctn id=\"Host\"/></route>\n", i_name, i_name);
 	}
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 	for (i = 0; i < ncuda; i++)
 	{
 		unsigned j;
@@ -2385,7 +2406,7 @@ flat_cuda:
 		}
 	}
 #endif
-	} /* defined(STARPU_HAVE_HWLOC) && defined(HAVE_CUDA_MEMCPY_PEER) */
+	} /* defined(STARPU_HAVE_HWLOC) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER) */
 	fprintf(f, "\n");
 #endif /* STARPU_USE_CUDA */
 
@@ -2426,7 +2447,7 @@ static void check_bus_platform_file(void)
 {
 	int res;
 
-	char path[256];
+	char path[PATH_LENGTH];
 	_starpu_simgrid_get_platform_path(4, path, sizeof(path));
 
 	res = access(path, F_OK);
@@ -2511,10 +2532,17 @@ double starpu_transfer_predict(unsigned src_node, unsigned dst_node, size_t size
 
 /* calculate save bandwidth and latency */
 /* bandwidth in MB/s - latency in µs */
-void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double bandwidth_read, double latency_write, double latency_read, unsigned node)
+void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double bandwidth_read, double latency_write, double latency_read, unsigned node, const char *name)
 {
 	unsigned int i, j;
 	double slowness_disk_between_main_ram, slowness_main_ram_between_node;
+	int print_stats = starpu_get_env_number_default("STARPU_BUS_STATS", 0);
+
+	if (print_stats)
+	{
+		fprintf(stderr, "\n#---------------------\n");
+		fprintf(stderr, "Data transfer speed for %s (node %u):\n", name, node);
+	}
 
 	/* save bandwith */
 	for(i = 0; i < STARPU_MAXNODES; ++i)
@@ -2539,6 +2567,9 @@ void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double band
 					slowness_main_ram_between_node = 0;
 				
 				bandwidth_matrix[i][j] = 1/(slowness_disk_between_main_ram+slowness_main_ram_between_node);
+
+				if (!isnan(bandwidth_matrix[i][j]) && print_stats)
+					fprintf(stderr,"%u -> %u: %.0f MB/s\n", i, j, bandwidth_matrix[i][j]);
 			}
 			else if (j == node) /* destination == disk */
 			{
@@ -2554,6 +2585,9 @@ void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double band
 					slowness_main_ram_between_node = 0;
 
 				bandwidth_matrix[i][j] = 1/(slowness_disk_between_main_ram+slowness_main_ram_between_node);
+
+				if (!isnan(bandwidth_matrix[i][j]) && print_stats)
+					fprintf(stderr,"%u -> %u: %.0f MB/s\n", i, j, bandwidth_matrix[i][j]);
 			}
 			else if (j > node || i > node) /* not affected by the node */
 			{
@@ -2574,10 +2608,16 @@ void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double band
 			else if (i == node) /* source == disk */
 			{
 				latency_matrix[i][j] = (latency_write+latency_matrix[STARPU_MAIN_RAM][j]);
+
+				if (!isnan(latency_matrix[i][j]) && print_stats)
+					fprintf(stderr,"%u -> %u: %.0f µs\n", i, j, latency_matrix[i][j]);
 			}
 			else if (j == node) /* destination == disk */
 			{
 				latency_matrix[i][j] = (latency_read+latency_matrix[i][STARPU_MAIN_RAM]);
+
+				if (!isnan(latency_matrix[i][j]) && print_stats)
+					fprintf(stderr,"%u -> %u: %.0f µs\n", i, j, latency_matrix[i][j]);
 			}
 			else if (j > node || i > node) /* not affected by the node */
 			{
@@ -2585,4 +2625,7 @@ void _starpu_save_bandwidth_and_latency_disk(double bandwidth_write, double band
 			}
 		}
 	}
+
+	if (print_stats)
+		fprintf(stderr, "\n#---------------------\n");
 }
