@@ -1,7 +1,8 @@
-/* StarPU --- Runtime system for heterogeneous multicore architectures. *
- * Copyright (C) 2009-2017  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017  CNRS
- * Copyright (C) 2014  INRIA
+/* StarPU --- Runtime system for heterogeneous multicore architectures.
+ *
+ * Copyright (C) 2011-2014,2017                           Inria
+ * Copyright (C) 2008-2018                                Université de Bordeaux
+ * Copyright (C) 2010-2018                                CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -249,14 +250,14 @@ static int worker_supports_direct_access(unsigned node, unsigned handling_node)
 			}
 			else
 				return 0;
-#elif defined(HAVE_CUDA_MEMCPY_PEER)
+#elif defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 			/* simgrid */
 			enum starpu_node_kind kind = starpu_node_get_kind(handling_node);
 			return kind == STARPU_CUDA_RAM;
-#else /* HAVE_CUDA_MEMCPY_PEER */
+#else /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 			/* Direct GPU-GPU transfers are not allowed in general */
 			return 0;
-#endif /* HAVE_CUDA_MEMCPY_PEER */
+#endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 		}
 		case STARPU_OPENCL_RAM:
 			return 0;
@@ -345,6 +346,15 @@ static int determine_request_path(starpu_data_handle_t handle,
 		return 1;
 	}
 
+	if (src_node < 0)
+	{
+		/* Will just initialize the destination */
+		STARPU_ASSERT(max_len >= 1);
+		src_nodes[0] = src_node; // ignored
+		dst_nodes[0] = dst_node;
+		return 1;
+	}
+
 	unsigned handling_node;
 	int link_is_valid = link_supports_direct_transfers(handle, src_node, dst_node, &handling_node);
 
@@ -402,7 +412,7 @@ static int determine_request_path(starpu_data_handle_t handle,
 		dst_nodes[0] = dst_node;
 		handling_nodes[0] = handling_node;
 
-#if !defined(HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+#if !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
 		STARPU_ASSERT(!(mode & STARPU_R) || starpu_node_get_kind(src_node) != STARPU_CUDA_RAM || starpu_node_get_kind(dst_node) != STARPU_CUDA_RAM);
 #endif
 
@@ -736,6 +746,28 @@ int _starpu_fetch_data_on_node(starpu_data_handle_t handle, int node, struct _st
 	if (cpt == STARPU_SPIN_MAXTRY)
 		_starpu_spin_lock(&handle->header_lock);
 
+	if (is_prefetch > 0)
+	{
+		unsigned src_node_mask = 0;
+
+		unsigned nnodes = starpu_memory_nodes_get_count();
+		for (node = 0; node < nnodes; node++)
+		{
+			if (handle->per_node[node].state != STARPU_INVALID)
+			{
+				/* we found a copy ! */
+				src_node_mask |= (1<<node);
+			}
+		}
+
+		if (src_node_mask == 0)
+		{
+			/* no valid copy, nothing to prefetch */
+			_starpu_spin_unlock(&handle->header_lock);
+			return 0;
+		}
+	}
+
 	if (!detached)
 	{
 		/* Take references which will be released by _starpu_release_data_on_node */
@@ -984,7 +1016,9 @@ int _starpu_fetch_task_input(struct _starpu_job *j)
 	{
 		starpu_data_handle_t handle = STARPU_TASK_GET_HANDLE(task, index);
 		enum starpu_data_access_mode mode = STARPU_TASK_GET_MODE(task, index);
-		int node = descrs[index].node;
+		int node = -1;
+		if (task->cl->specific_nodes)
+			node = STARPU_CODELET_GET_NODE(task->cl, index);
 		if (node == -1)
 			node = local_memory_node;
 
