@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009-2010, 2012-2016  Université de Bordeaux
- * Copyright (C) 2010, 2011, 2012, 2013, 2014, 2015, 2017  CNRS
+ * Copyright (C) 2011-2014,2016                           Inria
+ * Copyright (C) 2009-2018                                Université de Bordeaux
+ * Copyright (C) 2010-2017                                CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -48,7 +49,7 @@ static int malloc_on_node_default_flags[STARPU_MAXNODES];
 
 /* This file is used for implementing "folded" allocation */
 #ifdef STARPU_SIMGRID
-#if SIMGRID_VERSION_MAJOR < 3 || (SIMGRID_VERSION_MAJOR == 3 && SIMGRID_VERSION_MINOR < 15)
+#if SIMGRID_VERSION < 31500 || SIMGRID_VERSION == 31559
 /* TODO: drop when simgrid 3.15 is reasonably largely used by people who need the feature */
 static int bogusfile = -1;
 static unsigned long _starpu_malloc_simulation_fold;
@@ -62,7 +63,7 @@ void starpu_malloc_set_align(size_t align)
 		_malloc_align = align;
 }
 
-#if (defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER))// || defined(STARPU_USE_OPENCL)
+#if (defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER))// || defined(STARPU_USE_OPENCL)
 struct malloc_pinned_codelet_struct
 {
 	void **ptr;
@@ -81,7 +82,7 @@ struct malloc_pinned_codelet_struct
 //}
 //#endif
 
-#if defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
 static void malloc_pinned_cuda_codelet(void *buffers[] STARPU_ATTRIBUTE_UNUSED, void *arg)
 {
 	struct malloc_pinned_codelet_struct *s = arg;
@@ -93,7 +94,7 @@ static void malloc_pinned_cuda_codelet(void *buffers[] STARPU_ATTRIBUTE_UNUSED,
 }
 #endif
 
-#if (defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER)) && !defined(STARPU_SIMGRID)// || defined(STARPU_USE_OPENCL)
+#if (defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER)) && !defined(STARPU_SIMGRID)// || defined(STARPU_USE_OPENCL)
 static struct starpu_perfmodel malloc_pinned_model =
 {
 	.type = STARPU_HISTORY_BASED,
@@ -124,7 +125,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 			{
 				size_t freed;
 				size_t reclaim = 2 * dim;
-				_STARPU_DEBUG("There is not enough memory left, we are going to reclaim %ld\n", reclaim);
+				_STARPU_DEBUG("There is not enough memory left, we are going to reclaim %ld\n", (long)reclaim);
 				_STARPU_TRACE_START_MEMRECLAIM(STARPU_MAIN_RAM,0);
 				freed = _starpu_memory_reclaim_generic(STARPU_MAIN_RAM, 0, reclaim);
 				_STARPU_TRACE_END_MEMRECLAIM(STARPU_MAIN_RAM,0);
@@ -155,7 +156,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 		if (_starpu_can_submit_cuda_task())
 		{
 #ifdef STARPU_USE_CUDA
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 			cudaError_t cures;
 			cures = cudaHostAlloc(A, dim, cudaHostAllocPortable);
 			if (STARPU_UNLIKELY(cures))
@@ -183,6 +184,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 			task->callback_func = NULL;
 			task->cl = &malloc_pinned_cl;
 			task->cl_arg = &s;
+			task->type = STARPU_TASK_TYPE_INTERNAL;
 
 			task->synchronous = 1;
 
@@ -191,7 +193,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 			push_res = _starpu_task_submit_internally(task);
 			STARPU_ASSERT(push_res != -ENODEV);
 			goto end;
-#endif /* HAVE_CUDA_MEMCPY_PEER */
+#endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 #endif /* STARPU_USE_CUDA */
 		}
 //		else if (_starpu_can_submit_opencl_task())
@@ -214,6 +216,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 //			task->cl = &malloc_pinned_cl;
 //			task->cl_arg = &s;
 //			task->synchronous = 1;
+//			task->type = STARPU_TASK_TYPE_INTERNAL;
 //
 //			_starpu_exclude_task_from_dag(task);
 //
@@ -228,7 +231,7 @@ int starpu_malloc_flags(void **A, size_t dim, int flags)
 #ifdef STARPU_SIMGRID
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
 	{
-#if SIMGRID_VERSION_MAJOR > 3 || (SIMGRID_VERSION_MAJOR == 3 && SIMGRID_VERSION_MINOR >= 15)
+#if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
 		*A = SMPI_SHARED_MALLOC(dim);
 #else
 		/* TODO: drop when simgrid 3.15 is reasonably largely used by people who need the feature */
@@ -344,7 +347,7 @@ int starpu_malloc(void **A, size_t dim)
 	return starpu_malloc_flags(A, dim, STARPU_MALLOC_PINNED);
 }
 
-#if defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
 static void free_pinned_cuda_codelet(void *buffers[] STARPU_ATTRIBUTE_UNUSED, void *arg)
 {
 	cudaError_t cures;
@@ -363,7 +366,7 @@ static void free_pinned_cuda_codelet(void *buffers[] STARPU_ATTRIBUTE_UNUSED, vo
 //}
 //#endif
 
-#if defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID) // || defined(STARPU_USE_OPENCL)
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID) // || defined(STARPU_USE_OPENCL)
 static struct starpu_perfmodel free_pinned_model =
 {
 	.type = STARPU_HISTORY_BASED,
@@ -389,7 +392,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 		if (_starpu_can_submit_cuda_task())
 		{
 #ifdef STARPU_USE_CUDA
-#ifndef HAVE_CUDA_MEMCPY_PEER
+#ifndef STARPU_HAVE_CUDA_MEMCPY_PEER
 			if (!_starpu_is_initialized())
 			{
 #endif
@@ -400,7 +403,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 				if (STARPU_UNLIKELY(err))
 					STARPU_CUDA_REPORT_ERROR(err);
 				goto out;
-#ifndef HAVE_CUDA_MEMCPY_PEER
+#ifndef STARPU_HAVE_CUDA_MEMCPY_PEER
 			}
 			else
 			{
@@ -415,6 +418,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 				task->cl = &free_pinned_cl;
 				task->cl_arg = A;
 				task->synchronous = 1;
+				task->type = STARPU_TASK_TYPE_INTERNAL;
 
 				_starpu_exclude_task_from_dag(task);
 
@@ -422,7 +426,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 				STARPU_ASSERT(push_res != -ENODEV);
 				goto out;
 			}
-#endif /* HAVE_CUDA_MEMCPY_PEER */
+#endif /* STARPU_HAVE_CUDA_MEMCPY_PEER */
 #endif /* STARPU_USE_CUDA */
 		}
 //	else if (_starpu_can_submit_opencl_task())
@@ -439,6 +443,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 //		task->cl = &free_pinned_cl;
 //		task->cl_arg = A;
 //		task->synchronous = 1;
+//		task->type = STARPU_TASK_TYPE_INTERNAL;
 //
 //		_starpu_exclude_task_from_dag(task);
 //
@@ -453,7 +458,7 @@ int starpu_free_flags(void *A, size_t dim, int flags)
 #ifdef STARPU_SIMGRID
 	if (flags & STARPU_MALLOC_SIMULATION_FOLDED)
 	{
-#if SIMGRID_VERSION_MAJOR > 3 || (SIMGRID_VERSION_MAJOR == 3 && SIMGRID_VERSION_MINOR >= 15)
+#if SIMGRID_VERSION >= 31500 && SIMGRID_VERSION != 31559
 		SMPI_SHARED_FREE(A);
 #else
 		/* TODO: drop when simgrid 3.15 is reasonably largely used by people who need the feature */
@@ -515,7 +520,7 @@ _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 		case STARPU_CPU_RAM:
 		{
 			starpu_malloc_flags((void**) &addr, size,
-#if defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
 					/* without memcpy_peer, we can not
 					 * allocated pinned memory, since it
 					 * requires waiting for a task, and we
@@ -550,7 +555,7 @@ _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 			struct _starpu_worker *worker = _starpu_get_local_worker_key();
 			unsigned devid = _starpu_memory_node_get_devid(dst_node);
 			if (!worker || worker->arch != STARPU_CUDA_WORKER || worker->devid != devid)
-#if defined(HAVE_CUDA_MEMCPY_PEER)
+#if defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 				starpu_cuda_set_device(devid);
 #else
 				STARPU_ASSERT_MSG(0, "CUDA peer access is not available with this version of CUDA");
@@ -594,8 +599,8 @@ _starpu_malloc_on_node(unsigned dst_node, size_t size, int flags)
 				{
 					addr = (uintptr_t)ptr;
 				}
-				break;
 #endif
+			break;
 		}
 #endif
 	        case STARPU_DISK_RAM:
@@ -639,7 +644,7 @@ _starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int f
 	{
 		case STARPU_CPU_RAM:
 			starpu_free_flags((void*)addr, size,
-#if defined(STARPU_USE_CUDA) && !defined(HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
+#if defined(STARPU_USE_CUDA) && !defined(STARPU_HAVE_CUDA_MEMCPY_PEER) && !defined(STARPU_SIMGRID)
 					flags & ~STARPU_MALLOC_PINNED
 #else
 					flags
@@ -660,7 +665,7 @@ _starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int f
 			struct _starpu_worker *worker = _starpu_get_local_worker_key();
 			unsigned devid = _starpu_memory_node_get_devid(dst_node);
 			if (!worker || worker->arch != STARPU_CUDA_WORKER || worker->devid != devid)
-#if defined(HAVE_CUDA_MEMCPY_PEER)
+#if defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 				starpu_cuda_set_device(devid);
 #else
 				STARPU_ASSERT_MSG(0, "CUDA peer access is not available with this version of CUDA");
@@ -729,7 +734,7 @@ starpu_memory_pin(void *addr STARPU_ATTRIBUTE_UNUSED, size_t size STARPU_ATTRIBU
 {
 	if (STARPU_MALLOC_PINNED && disable_pinning <= 0 && STARPU_RUNNING_ON_VALGRIND == 0)
 	{
-#if defined(STARPU_USE_CUDA) && defined(HAVE_CUDA_MEMCPY_PEER)
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 		if (cudaHostRegister(addr, size, cudaHostRegisterPortable) != cudaSuccess)
 			return -1;
 #endif
@@ -742,7 +747,7 @@ starpu_memory_unpin(void *addr STARPU_ATTRIBUTE_UNUSED, size_t size STARPU_ATTRI
 {
 	if (STARPU_MALLOC_PINNED && disable_pinning <= 0 && STARPU_RUNNING_ON_VALGRIND == 0)
 	{
-#if defined(STARPU_USE_CUDA) && defined(HAVE_CUDA_MEMCPY_PEER)
+#if defined(STARPU_USE_CUDA) && defined(STARPU_HAVE_CUDA_MEMCPY_PEER)
 		if (cudaHostUnregister(addr) != cudaSuccess)
 			return -1;
 #endif
@@ -772,6 +777,10 @@ starpu_memory_unpin(void *addr STARPU_ATTRIBUTE_UNUSED, size_t size STARPU_ATTRI
  */
 #define CHUNK_ALLOC_MIN (16*1024)
 
+/* Don't really deallocate chunks unless we have more than this many chunks
+ * which are completely free. */
+#define CHUNKS_NFREE 4
+
 /* Number of blocks */
 #define CHUNK_NBLOCKS (CHUNK_SIZE/CHUNK_ALLOC_MIN)
 
@@ -813,7 +822,7 @@ _starpu_malloc_init(unsigned dst_node)
 	disable_pinning = starpu_get_env_number("STARPU_DISABLE_PINNING");
 	malloc_on_node_default_flags[dst_node] = STARPU_MALLOC_PINNED | STARPU_MALLOC_COUNT;
 #ifdef STARPU_SIMGRID
-#if SIMGRID_VERSION_MAJOR < 3 || (SIMGRID_VERSION_MAJOR == 3 && SIMGRID_VERSION_MINOR < 15)
+#if SIMGRID_VERSION < 31500 || SIMGRID_VERSION == 31559
 	/* Reasonably "costless" */
 	_starpu_malloc_simulation_fold = starpu_get_env_number_default("STARPU_MALLOC_SIMULATION_FOLD", 1) << 20;
 #endif
@@ -1036,7 +1045,7 @@ starpu_free_on_node_flags(unsigned dst_node, uintptr_t addr, size_t size, int fl
 	{
 		/* This chunk is now empty, but avoid chunk free/alloc
 		 * ping-pong by keeping some of these.  */
-		if (nfreechunks[dst_node] >= 1)
+		if (nfreechunks[dst_node] >= CHUNKS_NFREE)
 		{
 			/* We already have free chunks, release this one */
 			_starpu_free_on_node_flags(dst_node, chunk->base, CHUNK_SIZE, flags);
