@@ -1,6 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2011  Institut National de Recherche en Informatique et Automatique
+ * Copyright (C) 2011-2013                                Inria
+ * Copyright (C) 2012-2015,2017                           CNRS
+ * Copyright (C) 2013,2015                                Université de Bordeaux
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -13,10 +15,8 @@
  *
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
+
 #include <starpu.h>
-#ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-#endif
 
 /* XXX Why cant we dereference a handle without this one ? */
 #include <core/sched_policy.h>
@@ -27,7 +27,7 @@
 #include "../../helper.h"
 
 /*
- * This is definitely note thrad-safe.
+ * This is definitely note thread-safe.
  */
 static struct test_config *current_config;
 
@@ -35,20 +35,6 @@ static struct test_config *current_config;
 - OpenCL to OpenCL support
 */
 
-/*
- * Users do not know about this enum. They only know that SUCCESS is 0, and
- * FAILURE is 1. Therefore, the values of SUCCESS and FAILURE shall not be
- * changed.
- */
-enum exit_code
-{
-	SUCCESS                 = 0,
-	FAILURE                 = 1,
-	UNTESTED                = 2,
-	TASK_CREATION_FAILURE   = 3,
-	TASK_SUBMISSION_FAILURE = 4
-};
-
 static char *
 enum_to_string(int exit_code)
 {
@@ -91,6 +77,12 @@ struct data_interface_test_summary
 	int cpu_to_opencl_async;
 	int opencl_to_cpu_async;
 #endif
+#ifdef STARPU_USE_MIC
+	int cpu_to_mic;
+	int mic_to_cpu;
+	int cpu_to_mic_async;
+	int mic_to_cpu_async;
+#endif
 
 	/* Other stuff */
 	int compare;
@@ -122,6 +114,12 @@ void data_interface_test_summary_print(FILE *f,
 	FPRINTF(f, "\tOpenCl -> CPU    : %s\n",
 		enum_to_string(s->opencl_to_cpu_async));
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	FPRINTF(f, "\tCPU	-> MIC : %s\n",
+		enum_to_string(s->cpu_to_mic_async));
+	FPRINTF(f, "\tMIC	-> CPU : %s\n",
+		enum_to_string(s->mic_to_cpu_async));
+#endif
 
 	FPRINTF(f, "Synchronous :\n");
 #ifdef STARPU_USE_CUDA
@@ -138,6 +136,12 @@ void data_interface_test_summary_print(FILE *f,
 	FPRINTF(f, "\tOpenCl -> CPU    : %s\n",
 		enum_to_string(s->opencl_to_cpu));
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	FPRINTF(f, "\tCPU	-> MIC : %s\n",
+		enum_to_string(s->cpu_to_mic));
+	FPRINTF(f, "\tMIC	-> CPU : %s\n",
+		enum_to_string(s->mic_to_cpu));
+#endif
 #ifdef STARPU_USE_CPU
 	FPRINTF(f, "CPU    -> CPU    : %s\n",
 		enum_to_string(s->cpu_to_cpu));
@@ -156,7 +160,6 @@ data_interface_test_summary_success(data_interface_test_summary *s)
 
 enum operation
 {
-	CPU_TO_CPU,
 #ifdef STARPU_USE_CUDA
 	CPU_TO_CUDA,
 	CUDA_TO_CUDA,
@@ -164,8 +167,13 @@ enum operation
 #endif /* !STARPU_USE_CUDA */
 #ifdef STARPU_USE_OPENCL
 	CPU_TO_OPENCL,
-	OPENCL_TO_CPU
+	OPENCL_TO_CPU,
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	CPU_TO_MIC,
+	MIC_TO_CPU,
+#endif
+	CPU_TO_CPU
 };
 
 static int*
@@ -191,8 +199,14 @@ get_field(struct data_interface_test_summary *s, int async, enum operation op)
 	case OPENCL_TO_CPU:
 		return async?&s->opencl_to_cpu_async:&s->opencl_to_cpu;
 #endif /* !STARPU_USE_OPENCL */
-		default:
-			STARPU_ASSERT(0);
+#ifdef STARPU_USE_MIC
+	case CPU_TO_MIC:
+		return async?&s->cpu_to_mic_async:&s->cpu_to_mic;
+	case MIC_TO_CPU:
+		return async?&s->mic_to_cpu_async:&s->mic_to_cpu;
+#endif
+	default:
+		STARPU_ABORT();
 	}
 	/* that instruction should never be reached */
 	return NULL;
@@ -222,7 +236,7 @@ set_field(struct data_interface_test_summary *s, int async,
 			*field = TASK_SUBMISSION_FAILURE;
 			break;
 		default:
-			STARPU_ASSERT(0);
+			STARPU_ABORT();
 	}
 }
 
@@ -246,13 +260,18 @@ static struct data_interface_test_summary summary =
 	.cpu_to_opencl_async   = UNTESTED,
 	.opencl_to_cpu_async   = UNTESTED,
 #endif
+#ifdef STARPU_USE_MIC
+	.cpu_to_mic            = UNTESTED,
+	.mic_to_cpu            = UNTESTED,
+	.cpu_to_mic_async      = UNTESTED,
+	.mic_to_cpu_async      = UNTESTED,
+#endif
 #ifdef STARPU_USE_CPU
 	.handle_to_pointer     = UNTESTED,
 #endif
 	.success               = SUCCESS
 };
 
-
 /*
  * This variable has to be either -1 or 1.
  * The kernels should check that the ith value stored in the data interface is
@@ -265,7 +284,7 @@ static int factor = -1;
  * Note that the global variable <current_config> is heavily used here.
  * Arguments :
  *	- taskp : a pointer to a valid task
- *	- type : STARPU_{CPU,CUDA,OPENCL}_WORKER. Gordon is not supported.
+ *	- type : STARPU_{CPU,CUDA,OPENCL}_WORKER.
  *      - id   : -1 if you dont care about the device where the task will be
  *		 executed, as long as it has the right type.
  *		 >= 0 if you want to make sure the task will be executed on the
@@ -275,7 +294,7 @@ static int factor = -1;
  *	0 : success.
  */
 static int
-create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
+create_task(struct starpu_task **taskp, enum starpu_worker_archtype type, int id)
 {
 	static int cpu_workers[STARPU_MAXCPUS];
 #ifdef STARPU_USE_CUDA
@@ -284,6 +303,9 @@ create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
 #ifdef STARPU_USE_OPENCL
 	static int opencl_workers[STARPU_MAXOPENCLDEVS];
 #endif
+#ifdef STARPU_USE_MIC
+	static int mic_workers[STARPU_MAXMICDEVS];
+#endif
 
 	static int n_cpus = -1;
 #ifdef STARPU_USE_CUDA
@@ -292,11 +314,16 @@ create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
 #ifdef STARPU_USE_OPENCL
 	static int n_opencls = -1;
 #endif
+#ifdef STARPU_USE_MIC
+	static int n_mics = -1;
+#endif
 
 	if (n_cpus == -1) /* First time here */
 	{
-		/* XXX Dont check them all at once. */
-		/* XXX Error checking */
+		/* We do not check the return values of the calls to
+		 * starpu_worker_get_ids_by_type now, because it is simpler to
+		 * detect a problem in the switch that comes right after this 
+		 * block of code. */
 		n_cpus = starpu_worker_get_ids_by_type(STARPU_CPU_WORKER,
 							cpu_workers,
 							STARPU_MAXCPUS);
@@ -309,6 +336,11 @@ create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
 		n_opencls = starpu_worker_get_ids_by_type(STARPU_OPENCL_WORKER,
 							opencl_workers,
 							STARPU_MAXOPENCLDEVS);
+#endif
+#ifdef STARPU_USE_MIC
+		n_mics = starpu_worker_get_ids_by_type(STARPU_MIC_WORKER,
+							mic_workers,
+							STARPU_MAXMICDEVS);
 #endif
 	}
 
@@ -362,7 +394,21 @@ create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
 			cl.opencl_funcs[0] = current_config->opencl_func;
 			break;
 #endif /* ! STARPU_USE_OPENCL */
-		case STARPU_GORDON_WORKER: /* Not supported */
+#ifdef STARPU_USE_MIC
+		case STARPU_MIC_WORKER:
+		if (id != -1)
+		{
+			if (id >= n_mics)
+			{
+				FPRINTF(stderr, "Not enough MIC workers\n");
+				return -ENODEV;
+			}
+			workerid = mic_workers[id];
+		}
+		cl.where = STARPU_MIC;
+		cl.cpu_funcs_name[0] = current_config->cpu_func_name;
+		break;
+#endif
 		default:
 			return -ENODEV;
 	}
@@ -380,7 +426,7 @@ create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
 	}
 	factor = -factor;
 	task->cl_arg = &factor;
-	task->cl_arg_size = sizeof(&factor);
+	task->cl_arg_size = sizeof(factor);
 
 	*taskp = task;
 	return 0;
@@ -390,7 +436,7 @@ create_task(struct starpu_task **taskp, enum starpu_archtype type, int id)
  * <device1>_to_<device2> functions.
  * They all create and submit a task that has to be executed on <device2>,
  * forcing a copy between <device1> and <device2>.
- * XXX : could we sometimes use starp_insert_task() ? It seems hars because we
+ * XXX : could we sometimes use starpu_task_insert() ? It seems hars because we
  * need to set the execute_on_a_specific_worker field...
  */
 #ifdef STARPU_USE_CUDA
@@ -408,11 +454,11 @@ ram_to_cuda(void)
 	if (err != 0)
 		return TASK_SUBMISSION_FAILURE;
 
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	return current_config->copy_failed;
 }
 
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 static enum exit_code
 cuda_to_cuda(void)
 {
@@ -427,7 +473,7 @@ cuda_to_cuda(void)
 	if (err != 0)
 		return TASK_SUBMISSION_FAILURE;
 
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	return current_config->copy_failed;
 }
 #endif
@@ -446,14 +492,14 @@ cuda_to_ram(void)
 	if (err != 0)
 		return TASK_SUBMISSION_FAILURE;
 
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	return current_config->copy_failed;
 }
 #endif /* !STARPU_USE_CUDA */
 
 #ifdef STARPU_USE_OPENCL
 static enum exit_code
-ram_to_opencl()
+ram_to_opencl(void)
 {
 	int err;
 	struct starpu_task *task;
@@ -466,12 +512,12 @@ ram_to_opencl()
 	if (err != 0)
 		return TASK_SUBMISSION_FAILURE;
 
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	return current_config->copy_failed;
 }
 
 static enum exit_code
-opencl_to_ram()
+opencl_to_ram(void)
 {
 	int err;
 	struct starpu_task *task;
@@ -484,10 +530,48 @@ opencl_to_ram()
 	if (err != 0)
 		return TASK_SUBMISSION_FAILURE;
 
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	return current_config->copy_failed;
 }
 #endif /* !STARPU_USE_OPENCL */
+
+#ifdef STARPU_USE_MIC
+static enum exit_code
+ram_to_mic()
+{
+	int err;
+	struct starpu_task *task;
+
+	err = create_task(&task, STARPU_MIC_WORKER, 0);
+	if (err != 0)
+		return TASK_CREATION_FAILURE;
+
+	err = starpu_task_submit(task);
+	if (err != 0)
+		return TASK_SUBMISSION_FAILURE;
+
+	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	return current_config->copy_failed;
+}
+
+static enum exit_code
+mic_to_ram()
+{
+	int err;
+	struct starpu_task *task;
+
+	err = create_task(&task, STARPU_CPU_WORKER, -1);
+	if (err != 0)
+		return TASK_CREATION_FAILURE;
+	
+	err = starpu_task_submit(task);
+	if (err != 0)
+		return TASK_SUBMISSION_FAILURE;
+
+	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	return current_config->copy_failed;
+}
+#endif
 /* End of the <device1>_to_<device2> functions. */
 
 #ifdef STARPU_USE_CUDA
@@ -502,7 +586,7 @@ run_cuda(int async)
 	if (err != SUCCESS)
 		return;
 
-#ifdef HAVE_CUDA_MEMCPY_PEER
+#ifdef STARPU_HAVE_CUDA_MEMCPY_PEER
 	if (starpu_cuda_worker_get_count() >= 2)
 	{
 		err = cuda_to_cuda();
@@ -516,7 +600,7 @@ run_cuda(int async)
 	}
 #else
 	summary.cuda_to_cuda_async = UNTESTED;
-#endif /* !HAVE_CUDA_MEMCPY_PEER */
+#endif /* !STARPU_HAVE_CUDA_MEMCPY_PEER */
 
 #ifdef STARPU_USE_CPU
 	err = cuda_to_ram();
@@ -546,6 +630,24 @@ run_opencl(int async)
 }
 #endif /* !STARPU_USE_OPENCL */
 
+#ifdef STARPU_USE_MIC
+static void
+run_mic(int async)
+{
+	int err;
+
+	err = ram_to_mic();
+	set_field(&summary, async, CPU_TO_MIC, err);
+	if (err != SUCCESS)
+		return;
+
+#ifdef STARPU_USE_CPU
+	err = mic_to_ram();
+	set_field(&summary, async, MIC_TO_CPU, err);
+#endif
+}
+#endif /* STARPU_USE_PIC */
+
 #ifdef STARPU_USE_CPU
 /* Valid data should be in RAM before calling this function */
 static void
@@ -560,9 +662,12 @@ ram_to_ram(void)
 	dst = *current_config->dummy_handle;
 
 	/* We do not care about the nodes */
-	src_interface = starpu_data_get_interface_on_node(src, 0);
-	dst_interface = starpu_data_get_interface_on_node(dst, 0);
-	src->ops->copy_methods->ram_to_ram(src_interface, 0, dst_interface, 0);
+	src_interface = starpu_data_get_interface_on_node(src, STARPU_MAIN_RAM);
+	dst_interface = starpu_data_get_interface_on_node(dst, STARPU_MAIN_RAM);
+	if (src->ops->copy_methods->ram_to_ram)
+		src->ops->copy_methods->ram_to_ram(src_interface, STARPU_MAIN_RAM, dst_interface, STARPU_MAIN_RAM);
+	else
+		src->ops->copy_methods->any_to_any(src_interface, STARPU_MAIN_RAM, dst_interface, STARPU_MAIN_RAM, NULL);
 
 	err = create_task(&task, STARPU_CPU_WORKER, -1);
 	if (err != 0)
@@ -578,7 +683,7 @@ ram_to_ram(void)
 		goto out;
 	}
 
-	FPRINTF(stderr, "[%s] : %d\n", __func__, current_config->copy_failed);
+	FPRINTF(stderr, "[%s] : %d\n", __starpu_func__, current_config->copy_failed);
 	err = current_config->copy_failed;
 
 out:
@@ -590,7 +695,8 @@ static void
 run_async(void)
 {
 	int async = starpu_asynchronous_copy_disabled();
-	if (async == 1) {
+	if (async == 1)
+	{
 		FPRINTF(stderr, "Asynchronous copies have been disabled\n");
 		return;
 	}
@@ -600,6 +706,9 @@ run_async(void)
 #ifdef STARPU_USE_OPENCL
 	run_opencl(1);
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	run_mic(1);
+#endif
 }
 
 static void
@@ -624,6 +733,10 @@ run_sync(void)
 	new_copy_methods.ram_to_opencl_async = NULL;
 	new_copy_methods.opencl_to_ram_async = NULL;
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	new_copy_methods.ram_to_mic_async = NULL;
+	new_copy_methods.mic_to_ram_async = NULL;
+#endif
 
 	handle->ops->copy_methods = &new_copy_methods;
 
@@ -633,6 +746,9 @@ run_sync(void)
 #ifdef STARPU_USE_OPENCL
 	run_opencl(0);
 #endif /* !STARPU_USE_OPENCL */
+#ifdef STARPU_USE_MIC
+	run_mic(0);
+#endif
 
 	handle->ops->copy_methods = old_copy_methods;
 }
@@ -646,8 +762,8 @@ compare(void)
 
 	handle = *current_config->handle;
 	dummy_handle = *current_config->dummy_handle;
-	interface_a = starpu_data_get_interface_on_node(handle, 0);
-	interface_b = starpu_data_get_interface_on_node(dummy_handle, 0);
+	interface_a = starpu_data_get_interface_on_node(handle, STARPU_MAIN_RAM);
+	interface_b = starpu_data_get_interface_on_node(dummy_handle, STARPU_MAIN_RAM);
 
 	err = handle->ops->compare(interface_a, interface_b);
 
@@ -707,6 +823,9 @@ load_conf(struct test_config *config)
 #endif
 #ifdef STARPU_USE_OPENCL
 	    !config->opencl_func ||
+#endif
+#ifdef STARPU_USE_MIC
+		!config->cpu_func_name ||
 #endif
 	    !config->handle)
 	{
