@@ -1,7 +1,8 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2009, 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010, 2011, 2012  Centre National de la Recherche Scientifique
+ * Copyright (C) 2012                                     Inria
+ * Copyright (C) 2009-2014,2016                           Université de Bordeaux
+ * Copyright (C) 2010-2013,2015-2017                      CNRS
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +16,6 @@
  * See the GNU Lesser General Public License in COPYING.LGPL for more details.
  */
 
-#include <config.h>
 #include <stdio.h>
 #include <unistd.h>
 #include <errno.h>
@@ -23,21 +23,23 @@
 #include <stdlib.h>
 #include "../helper.h"
 
+/*
+ * Test initializing a buffer with a task, then printing it with another task
+ */
+
 #define VECTORSIZE	1024
 
 starpu_data_handle_t v_handle;
 
 #ifdef STARPU_USE_OPENCL
-#include <starpu_opencl.h>
-
-static void opencl_codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+static void opencl_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
 	cl_mem buf = (cl_mem)STARPU_VECTOR_GET_DEV_HANDLE(descr[0]);
         char ptr = 42;
         cl_command_queue queue;
-        int id = starpu_worker_get_id();
+        int id = starpu_worker_get_id_check();
         int devid = starpu_worker_get_devid(id);
 
         starpu_opencl_get_queue(devid, &queue);
@@ -46,26 +48,24 @@ static void opencl_codelet_null(void *descr[], __attribute__ ((unused)) void *_a
 #endif
 
 #ifdef STARPU_USE_CUDA
-static void cuda_codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+static void cuda_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
 
-	cudaMemset(buf, 42, 1);
+	cudaMemsetAsync(buf, 42, 1, starpu_cuda_get_local_stream());
 }
 #endif
 
-static void cpu_codelet_null(void *descr[], __attribute__ ((unused)) void *_args)
+void cpu_codelet_null(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
-	STARPU_SKIP_IF_VALGRIND;
-
 	char *buf = (char *)STARPU_VECTOR_GET_PTR(descr[0]);
 
 	*buf = 42;
 }
 
-static void display_var(void *descr[], __attribute__ ((unused)) void *_args)
+void display_var(void *descr[], STARPU_ATTRIBUTE_UNUSED void *_args)
 {
 	STARPU_SKIP_IF_VALGRIND;
 
@@ -79,22 +79,24 @@ static void display_var(void *descr[], __attribute__ ((unused)) void *_args)
 
 static struct starpu_codelet cl =
 {
-	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_funcs = {cpu_codelet_null, NULL},
+	.cpu_funcs = {cpu_codelet_null},
 #ifdef STARPU_USE_CUDA
-	.cuda_funcs = {cuda_codelet_null, NULL},
+	.cuda_funcs = {cuda_codelet_null},
+	.cuda_flags = {STARPU_CUDA_ASYNC},
 #endif
 #ifdef STARPU_USE_OPENCL
-	.opencl_funcs = {opencl_codelet_null, NULL},
+	.opencl_funcs = {opencl_codelet_null},
+	.opencl_flags = {STARPU_OPENCL_ASYNC},
 #endif
+	.cpu_funcs_name = {"cpu_codelet_null"},
 	.nbuffers = 1,
 	.modes = {STARPU_W}
 };
 
 static struct starpu_codelet display_cl =
 {
-	.where = STARPU_CPU,
-	.cpu_funcs = {display_var, NULL},
+	.cpu_funcs = {display_var},
+	.cpu_funcs_name = {"display_var"},
 	.nbuffers = 1,
 	.modes = {STARPU_R}
 };
@@ -104,7 +106,7 @@ int main(int argc, char **argv)
 {
 	int ret;
 
-	ret = starpu_init(NULL);
+	ret = starpu_initialize(NULL, &argc, &argv);
 	if (ret == -ENODEV) return STARPU_TEST_SKIPPED;
 	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
